import configparser, os, sys, tkinter, webbrowser
from tkinter import filedialog, messagebox, simpledialog
from Card import *
from Global import *
from Point import Point
DATAFILE_EXTENSION = '.txt'
FILETYPES = {('テキストファイル', '*.txt')}
MENU_FILE_SAVE = '保存(S)'

class IdeaCards:
  _dataFilename:str
  @property
  def dataFilename(self):
    return self._dataFilename
  @dataFilename.setter
  def dataFilename(self, value:str):
    self._dataFilename = value
    if value == '':
      root.title(self.__class__.__name__)
      self.menuFile.entryconfigure(
        MENU_FILE_SAVE, state='disabled')
    else:
      s = os.path.basename(value)
      root.title(s)
      self.menuFile.entryconfigure(
        MENU_FILE_SAVE, state='normal')
      self.directory = os.path.dirname(value)
      
  def __init__(self, root:tkinter.Tk):
    root.title(self.__class__.__name__)
    clientX = '200'
    clientY = '200'
    clientHeight = '100'
    clientWidth = '300'
    self.directory = (os.getenv('HOMEDRIVE') +
      os.getenv('HOMEPATH') + os.sep + 'Documents')
    self.iniFilename = os.path.abspath(
    __file__).replace('.py', '.ini')
    cp = configparser.ConfigParser()
    try:
      cp.read(self.iniFilename)
      clientX = cp['client']['x']
      clientY = cp['client']['y']
      clientHeight = cp['client']['height']
      clientWidth = cp['client']['width']
      dir = cp['file']['directory']
      if os.path.isdir(dir): self.directory = dir
    except:
      print(self.__class__.__name__ +
        ': Use default value(s)', file=sys.stderr)
    root.geometry(clientWidth + 'x' + clientHeight +
      '+' + clientX + '+' + clientY)

    root.option_add('*tearOff', tkinter.FALSE)
    menu = tkinter.Menu()
    self.menuFile = tkinter.Menu()
    menu.add_cascade(menu=self.menuFile,
      label='ファイル(F)', underline=5)
    self.menuFile.add_command(label='新規(N)', underline=3,
      command=self.menuFileNew, accelerator='Ctrl+N')
    self.menuFile.add_command(label='開く(O)', underline=3,
      command=self.menuFileOpen, accelerator='Ctrl+O')
    self.menuFile.add_command(label=MENU_FILE_SAVE, underline=3,
      command=self.menuFileSave, accelerator='Ctrl+S')
    self.menuFile.add_command(label='名前を付けて保存(A)',
      underline=9, command=self.menuFileSaveAs)
    self.menuFile.add_separator()
    self.menuFile.add_command(label='終了(X)', underline=3,
      command=self.menuFileExit, accelerator='Alt+F4')
    menuEdit = tkinter.Menu()
    menu.add_cascade(menu=menuEdit, label='編集(E)', underline=3)
    menuEdit.add_command(label='追加(A)', underline=3,
      command=self.menuEditAdd, accelerator='Ctrl+I')
    menuHelp = tkinter.Menu()
    menu.add_cascade(menu=menuHelp,
      label='ヘルプ(H)', underline=4)
    menuHelp.add_command(label='ヘルプファイルを開く(O)',
      underline=10, command=self.menuHelpOpenWeb)
    menuHelp.add_separator()
    menuHelp.add_command(label='バージョン情報(V)',
      underline=8, command=self.menuHelpVersion)
    root['menu'] = menu
    #root.bind_all('<Alt-F4>', self.menuFileExit)
    root.bind_all('<Control-Key-n>', self.menuFileNew)
    root.bind_all('<Control-Key-N>', self.menuFileNew)
    root.bind_all('<Control-Key-o>', self.menuFileOpen)
    root.bind_all('<Control-Key-O>', self.menuFileOpen)
    root.bind_all('<Control-Key-s>', self.menuFileSave)
    root.bind_all('<Control-Key-S>', self.menuFileSave)
    root.bind_all('<Control-Key-i>', self.menuEditAdd)
    root.bind_all('<Control-Key-I>', self.menuEditAdd)
    root.protocol('WM_DELETE_WINDOW', self.menuFileExit)
    root.bind('<Configure>', self.configure)

    self.canvas = tkinter.Canvas(root, bg='green')
    self.canvas.pack(expand=True, fill=tkinter.BOTH)
    self.canvas.bind('<ButtonPress-1>', self.b1Press)
    self.canvas.bind('<B1-Motion>', self.b1Motion)
    self.canvas.bind('<ButtonRelease-1>', self.b1Release)
    self.afterId = None
    self.b1PressP = Point()
    self.b1MotionP = Point()
    self.b1ReleaseP = Point()
    self.cards = list()
    self.cards.append(Card())
    self.dataFilename = ''

  def configure(self, event=None):
    self.update()

  def update(self, all:bool=False):
    if self.afterId != None: root.after_cancel(self.afterId)
    if all:
      self.canvas.delete('all')
      for c in self.cards: c.clearId()
    for c in self.cards: self.drawCard(c)
    self.afterId = root.after(100, self.update)

  def drawCard(self, c:Card):
    match c.mode:
      case 0 | 3:
        if c.rectId == 0:
          c.rectId =self.canvas.create_rectangle(
            c.x, c.y, c.x + c.width, c.y + c.height,
            width=CARD_BORDER_WIDTH, outline='black', 
            fill=CARD_FILL_COLOR)
        if c.textId == 0: self.drawText(c)
      case 1:
        self.canvas.delete(c.rectId) 
        c.rectId = self.canvas.create_rectangle(
          c.x + self.b1MotionP.x, c.y + self.b1MotionP.y, 
          c.x + self.b1MotionP.x + c.width,
          c.y + self.b1MotionP.y + c.height,
          width=CARD_BORDER_WIDTH, outline='blue',
          fill=CARD_FILL_COLOR)
        self.canvas.delete(c.textId)
        c.textId = self.canvas.create_text(
          c.x + self.b1MotionP.x + 10, 
          c.y + self.b1MotionP.y + 10, 
          anchor='nw', text=c.text, font=('', CARD_FONT_SIZE))
      case 2: 
        self.canvas.delete(c.rectId)
        newWidth = self.b1MotionP.x + c.width
        if newWidth < CARD_MIN_WIDTH:
          newWidth = CARD_MIN_WIDTH
        newHeight = self.b1MotionP.y + c.height
        if newHeight < CARD_MIN_HEIGHT:
          newHeight = CARD_MIN_HEIGHT
        c.rectId = self.canvas.create_rectangle(
          c.x, c.y, c.x + newWidth, c.y + newHeight,
          width=CARD_BORDER_WIDTH, outline='red', 
          fill=CARD_FILL_COLOR)
        self.canvas.delete(c.textId)
        self.drawText(c)

  def drawText(self, c:Card):
    c.textId = self.canvas.create_text(c.x + 10, c.y + 10, 
    anchor='nw', text=c.text, font=('', CARD_FONT_SIZE))

  def b1Press(self, event=None):
    self.b1PressP.setValue(event.x, event.y)
    for i in range(len(self.cards)-1, -1, -1):
      self.cards[i].setMode(self.b1PressP.x, self.b1PressP.y)
      if self.cards[i].mode != 0: break

  def b1Motion(self, event=None):
    self.b1MotionP.setValue(event.x - self.b1PressP.x,
      event.y - self.b1PressP.y)
    
  ## list11
  def b1Release(self, event=None):
    self.b1ReleaseP.setValue(event.x, event.y)
    for c in self.cards:
      match c.mode:
        case 1:
          c.x += self.b1MotionP.x
          c.y += self.b1MotionP.y
        case 2:
          c.width += self.b1MotionP.x
          c.height += self.b1MotionP.y
          if c.width < CARD_MIN_WIDTH:
            c.width = CARD_MIN_WIDTH
          if c.height < CARD_MIN_HEIGHT:
            c.height = CARD_MIN_HEIGHT
        case 3:
          s = simpledialog.askstring(
           '編集', '文字を編集してください\t\t',
            initialvalue=ToOneLine(c.text))
          if s == None: pass
          elif s == '':
            if messagebox.askyesno('確認', 'カードを削除しますか？'):
              self.cards.remove(c)
              self.update(all=True)
          else:
            c.text = ToMultiLine(s)
      if c.mode != 0:
        self.b1MotionP.setValue(0, 0)
        c.mode = 0
        self.redrawCard(c)
        break

  def redrawCard(self, c:Card):
    self.canvas.delete(c.rectId)
    self.canvas.delete(c.textId)
    c.clearId()
    self.update()
    
  ## list13
  def menuFileNew(self, event=None):
    self.cards.clear()
    self.update(all=True)
    self.dataFilename = ''
    
  def menuFileOpen(self, event=None):
    filename = filedialog.askopenfilename(
      filetypes=FILETYPES,
      initialdir=self.directory
    )
    if filename:
      s = ''
      f = open(filename, 'r')
      s = f.read()
      f.close()
      lines = s.splitlines()
      self.cards.clear()
      for i in range(1, len(lines), 5):
        c = Card()
        c.x = int(lines[i-1])
        c.y = int(lines[i])
        c.width = int(lines[i+1])
        c.height = int(lines[i+2])
        c.text = ToMultiLine(lines[i+3])
        self.cards.append(c)
      self.update(all=True)
      self.dataFilename = filename
      
  def menuFileSave(self, event=None):
    self.fileSave(self.dataFilename)
    
  def menuFileSaveAs(self, event=None):
    filename = filedialog.asksaveasfilename(
      filetypes=FILETYPES, initialdir=self.directory,
      initialfile=os.path.basename(self.dataFilename)
    )
    if filename:
      s = os.path.splitext(filename)
      if s[len(s)-1].lower() != DATAFILE_EXTENSION:
        filename += DATAFILE_EXTENSION
      self.fileSave(filename)
      
  def fileSave(self, saveFilename:str):
    if len(self.cards) < 1:
      messagebox.showwarning(self.__class__.__name__,
        '保存するデータがありません')
    else:
      s = ''
      for c in self.cards:
        s += (str(c.x) + '\n' + str(c.y) + '\n' +
          str(c.width) + '\n' + str(c.height) + '\n' +
          ToOneLine(c.text) + '\n')
      f = open(saveFilename, 'w')
      f.write(s)
      f.close()
      self.dataFilename = saveFilename

  def menuFileExit(self, event=None):
    cp = configparser.ConfigParser()
    cp['client'] = {
      'x': str(root.winfo_x()),
      'y': str(root.winfo_y()),
      'height': str(root.winfo_height()),
      'width': str(root.winfo_width())
    }
    cp['file'] = {'directory': self.directory}
    with open(self.iniFilename, 'w') as f:
      cp.write(f)
    root.destroy()

  def menuEditAdd(self, event=None):
    newCard = Card()
    for c in self.cards:
      if newCard.x == c.x and newCard.y == c.y:
        newCard.x += 10
        newCard.y += 10
    self.cards.append(newCard)
    self.update(all=True)

  def menuHelpOpenWeb(self):
    helpFilePath = os.path.dirname(
      __file__) + os.sep + 'help.htm'
    if os.path.isfile(helpFilePath):
      helpFilePath = 'file:///' + helpFilePath.replace(
        os.sep, '/')
      webbrowser.open(helpFilePath)
    else:
      messagebox.showerror(self.__class__.__name__,
        helpFilePath + 'がありません')
      
  def menuHelpVersion(self):
    s = self.__class__.__name__
    s += ' Version 0.01(2023/10/20)\n'
    s += '©2023 Hideo Harada\n'
    s += 'with Python ' + sys.version
    messagebox.showinfo(self.__class__.__name__, s)

root = tkinter.Tk()
IdeaCards(root)
root.mainloop()