import tkinter
from tkinter import simpledialog
from Card import *
from Global import *
from Point import Point

class OneCard:
  def __init__(self, root:tkinter.Tk):
    root.title(self.__class__.__name__)
    root.geometry('640x400')
    root.bind('<Configure>', self.configure)
    self.canvas = tkinter.Canvas(root, bg='green')
    self.canvas.pack(expand=True, fill=tkinter.BOTH)
    self.canvas.bind('<ButtonPress-1>', self.b1Press)
    self.canvas.bind('<B1-Motion>', self.b1Motion)
    self.canvas.bind('<ButtonRelease-1>', self.b1Release)
    self.afterId = None
    self.b1PressP = Point()
    self.b1MotionP = Point()
    self.b1ReleaseP = Point()
    self.mouseInfoId = 0
    self.card1 = Card()

  def configure(self, event=None):
    self.update()

  def update(self):
    if self.afterId != None: root.after_cancel(self.afterId)
    self.canvas.delete(self.mouseInfoId)
    self.mouseInfoId = self.canvas.create_text(
      10, 10, anchor='w', fill='white',
      text=str(self.b1PressP) + '->' +
      str(self.b1MotionP) + '->' + str(self.b1ReleaseP))
    self.drawCard(self.card1)
    self.afterId = root.after(100, self.update)

  def drawCard(self, c:Card):
    match c.mode:
      case 0 | 3:
        if c.rectId == 0:
          c.rectId =self.canvas.create_rectangle(
            c.x, c.y, c.x + c.width, c.y + c.height,
            width=CARD_BORDER_WIDTH, outline='black',
            fill=CARD_FILL_COLOR)
        if c.textId == 0: self.drawText(c)
      case 1:
        self.canvas.delete(c.rectId)
        c.rectId = self.canvas.create_rectangle(
          c.x + self.b1MotionP.x, c.y + self.b1MotionP.y,
          c.x + self.b1MotionP.x + c.width,
          c.y + self.b1MotionP.y + c.height,
          width=CARD_BORDER_WIDTH, outline='blue',
          fill=CARD_FILL_COLOR)
        self.canvas.delete(c.textId)
        c.textId = self.canvas.create_text(
          c.x + self.b1MotionP.x + 10,
          c.y + self.b1MotionP.y + 10, anchor='nw',
            text=c.text, font=('', CARD_FONT_SIZE))
      case 2:
        self.canvas.delete(c.rectId)
        newWidth = self.b1MotionP.x + c.width
        if newWidth < CARD_MIN_WIDTH:
          newWidth = CARD_MIN_WIDTH
        newHeight = self.b1MotionP.y + c.height
        if newHeight < CARD_MIN_HEIGHT:
          newHeight = CARD_MIN_HEIGHT
        c.rectId = self.canvas.create_rectangle(
          c.x, c.y, c.x + newWidth, c.y + newHeight,
          width=CARD_BORDER_WIDTH, outline='red',
          fill=CARD_FILL_COLOR)
        self.canvas.delete(c.textId)
        self.drawText(c)

  def drawText(self, c:Card):
    c.textId = self.canvas.create_text(c.x + 10, c.y + 10,
      anchor='nw', text=c.text, font=('', CARD_FONT_SIZE))
    
  def b1Press(self, event=None):
    self.b1PressP.setValue(event.x, event.y)
    self.card1.setMode(self.b1PressP.x, self.b1PressP.y)

  def b1Motion(self, event=None):
    self.b1MotionP.setValue(event.x - self.b1PressP.x,
      event.y - self.b1PressP.y)
    
  def b1Release(self, event=None):
    self.b1ReleaseP.setValue(event.x, event.y)
    match self.card1.mode:
      case 1:
        self.card1.x += self.b1MotionP.x
        self.card1.y += self.b1MotionP.y
      case 2:
        self.card1.width += self.b1MotionP.x
        if self.card1.width < CARD_MIN_WIDTH:
          self.card1.width = CARD_MIN_WIDTH
        self.card1.height += self.b1MotionP.y
        if self.card1.height < CARD_MIN_HEIGHT:
          self.card1.height = CARD_MIN_HEIGHT
      case 3:
        s = simpledialog.askstring(
          '編集', '文字を編集してください\t\t',
          initialvalue = ToOneLine(self.card1.text))
        if s != None:
          self.card1.text = ToMultiLine(s)
          self.update()
    if self.card1.mode != 0:
      self.b1MotionP.setValue(0, 0)
      self.card1.mode = 0
      self.redrawCard(self.card1)

  def redrawCard(self, c:Card):
    self.canvas.delete(c.rectId)
    self.canvas.delete(c.textId)
    c.clearId()
    self.update()

root = tkinter.Tk()
OneCard(root)
root.mainloop()